@@ -3,8 +3,17 @@ use Mojo::Base 'Mojolicious::Plugin::Authentication';
 use Mojo::Loader qw(load_class);
 use Mojo::Util qw(hmac_sha1_sum);
 use Hash::Merge qw( merge );
+use DBIx::POS::Sth;
 
-our $VERSION = '0.501';
+our $VERSION = '0.701';
+
+=pod
+
+=head1 VERSION
+
+0.701
+
+=cut
 
 my $pkg = __PACKAGE__;
 
@@ -23,22 +32,36 @@ has default => sub {
     namespace => $pkg,
     module => 'Access',
     fail_auth_cb => sub {
-      shift->render(format=>'txt', text=>"Access deny at auth step. Please sign in!!!\n");
+      shift->render(format=>'txt', text=>"Deny access at auth step. Please sign in.\n");
     },
     fail_access_cb => sub {
-      shift->render(format=>'txt', text=>"You don`t have access on this route (url, action) !!!\n");
+      shift->render(format=>'txt', text=>"You don`t have access on this route (url, action).\n");
     },
     import => [qw(load_user validate_user)],
+    pos => {
+      namespace => $pkg,
+      module => 'POS::Access',
+    },
   },
   admin => {
     namespace => $pkg,
     controller => 'Admin',
     prefix => lc($self->conf->{admin}{controller} || 'admin'),
     trust => hmac_sha1_sum('admin', $self->app->secrets->[0]),
+    pos => {
+      namespace => $pkg,
+      module => 'POS::Admin',
+    },
+  },
+  oauth => {
+    namespace => $pkg,
+    controller => 'OAuth2',
+    pos => {
+      namespace => $pkg,
+      module => 'POS::OAuth2',
+    },
+    fail_auth_cb => sub {shift->render(format=>'txt', text=>"@_")},
   },
-  pos => {namespace => $pkg, module => 'POS::Pg', },
-  sth => {namespace => $pkg, module => 'Sth', },
-  template => {schema => 'public'},
 }};
 
 has merge_conf => sub {#hashref
@@ -46,21 +69,6 @@ has merge_conf => sub {#hashref
   merge($self->conf, $self->default);
 };
 
-has pos => sub {# object DBIx::POS::Template
-  my $self = shift;
-  my $pos = $self->merge_conf->{'pos'};
-  my $class = $self->_class($pos);
-  $class->new;
-};
-
-has sth => sub {# object Sth
-  my $self = shift;
-  my $sth = $self->merge_conf->{'sth'};
-  my $class = $self->_class($sth);
-  my $template = $self->merge_conf->{template};
-  $class->new($self->dbh, $self->pos, %$template);
-};
-
 has access => sub {# object
   my $self = shift;
   my $access = $self->merge_conf->{'access'};
@@ -68,23 +76,49 @@ has access => sub {# object
   $class->import( @{$access->{import}});
   bless $access, $class;
   $access->{dbh} = $self->dbh;
-  $access->{sth} = $self->sth;
+  my $pos = $access->{pos};
+  $access->{sth} = DBIx::POS::Sth->new(
+    $self->dbh,
+    $self->_class($pos)->new($pos->{template} ? (template=>$pos->{template}) : ())
+  );
   $access->{app} = $self->app;
+  $access->{plugin} = $self;
   return $access->init;
 };
 
-has admin => sub {
+has admin => sub {# object
   my $self = shift;
   my $admin = $self->merge_conf->{'admin'};
   $admin->{module} ||= $admin->{controller};
   my $class = $self->_class($admin);
   bless $admin, $class;
   $admin->{dbh} = $self->dbh;
-  $admin->{sth} = $self->sth;
-  
+  my $pos = $admin->{pos};
+  $admin->{sth} = DBIx::POS::Sth->new(
+    $self->dbh,
+    $self->_class($pos)->new($pos->{template} ? (template=>$pos->{template}) : ()),
+  );
+  $admin->{plugin} = $self;
   return $admin->init;
 };
 
+has oauth => sub {
+  my $self = shift;
+  my $oauth = $self->merge_conf->{'oauth'};
+  my $class = $self->_class($oauth);
+  bless $oauth, $class;
+  $oauth->{dbh} = $self->dbh;
+  my $pos = $oauth->{pos};
+  $oauth->{sth} = DBIx::POS::Sth->new(
+    $self->dbh,
+    $self->_class($pos)->new($pos->{template} ? (template=>$pos->{template}) : ()),
+  );
+  $oauth->{app} = $self->app;
+  $oauth->{plugin} = $self;
+  #~ $oauth->{admin} = $self->admin;
+  return $oauth->init;
+};
+
 
 sub register {
   my $self = shift;
@@ -98,7 +132,8 @@ sub register {
   
   my $access = $self->access;
   
-  #~ die $self->app->dumper($access);
+  die "Plugin [Authentication] already loaded"
+    if $self->app->renderer->helpers->{'authenticate'};
   
   $self->SUPER::register($self->app, $self->merge_conf->{auth});
   
@@ -106,11 +141,17 @@ sub register {
   $access->apply_ns();
   $access->apply_route($_) for @{ $access->db_routes };
   
+  if ($self->conf->{oauth}) {
+    my $oauth = $self->oauth;
+    $access->apply_route($_) for $oauth->_routes;
+  }
+  
   if ($self->conf->{admin}) {
     my $admin = $self->admin;
     $access->apply_route($_) for $admin->self_routes;
   }
   
+  
   $self->app->helper('access', sub {$access});
   
   return $self, $access;
@@ -120,7 +161,10 @@ sub register {
 sub _class {
   my $self = shift;
   my $conf = shift;
-  my $class  = join '::', $conf->{namespace}, $conf->{module};
+  my $class  = join '::', $conf->{namespace}, $conf->{module} || $conf->{controller}
+    if $conf->{namespace};
+  $class ||= $conf->{module} || $conf->{controller};
+  
   my $e; $e = load_class($class)# success undef
     and die $e;
   return $class;
@@ -143,14 +187,15 @@ sub cond_access {# add_condition
     and return 1 # не проверяем доступ
     unless $args->{auth};
   
+  my $fail_auth_cb = $access->{fail_auth_cb};
+  
   # не авторизовался
-  #~ $app->log->debug("Access deny for non auth user")
   $self->deny_log($route, $args, $u)
-    and $access->{fail_auth_cb}->($c, )
+    and $c->$fail_auth_cb()
     and return undef
     unless $u;
   
-  #  получить все группы пользователя
+  #  получить все группы
   $access->load_user_roles($u);
   
   # допустить если {auth=>'only'}
@@ -163,8 +208,7 @@ sub cond_access {# add_condition
   if (ref $args eq 'CODE') {
     $args->($u, @_)
       or $self->deny_log($route, $args, $u)
-      and $access->{fail_auth_cb}->($c, )
-      #~ and $app->log->debug("Access deny user by callback condition")
+      and $c->$fail_auth_cb()
       and return undef;
     $app->log->debug(sprintf(qq[Access allow [%s] by callback condition],
       $route->pattern->unparsed,
@@ -190,7 +234,7 @@ sub cond_access {# add_condition
   # Access to non db route by role
   $args->{role}
     && $access->access_role($args->{role}, $id2)
-    && $app->log->debug(sprintf "Access allow [%s] by role [%s] joined roles=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] by role [%s]",
       $route->pattern->unparsed,
       $args->{role},
     )
@@ -202,8 +246,11 @@ sub cond_access {# add_condition
   if ($controller && !$namespace) {
     (load_class($_.'::'.$controller) or ($namespace = $_) and last) for @{ $app->routes->namespaces };
   }
+  
+  my $fail_access_cb = $access->{fail_access_cb};
+  
   $self->deny_log($route, $args, $u)
-    and $access->{fail_access_cb}->($c,)
+    and $c->$fail_access_cb()
     and return undef
     unless $controller && $namespace;# failed load class
 
@@ -241,7 +288,7 @@ sub cond_access {# add_condition
   
   my $action = $args->{action} || $route->pattern->defaults->{action}
     or $self->deny_log($route, $args, $u)
-    and $access->{fail_access_cb}->($c,)
+    and $c->$fail_access_cb()
     and return undef;
   
   $access->access_action($namespace, $controller, $action, $id2)
@@ -267,7 +314,7 @@ sub cond_access {# add_condition
     && return 1;
     
   $self->deny_log($route, $args, $u);
-  $access->{fail_access_cb}->($c,);
+  $c->$fail_access_cb();
   return undef;
 }
 
@@ -275,7 +322,7 @@ sub deny_log {
   my $self = shift;
   my ($route, $args, $u) = @_;
   my $app = $self->app;
-  $app->log->debug(sprintf "Access deny [%s] for user id=[%s]; args=[%s]; defaults=[%s]",
+  $app->log->debug(sprintf "Access deny [%s] for profile id=[%s]; args=[%s]; defaults=[%s]",
     $route->pattern->unparsed,
     $u ? $u->{id} : 'non auth',
     $app->dumper($args) =~ s/\s+//gr,
@@ -295,10 +342,6 @@ sub deny_log {
 
 ¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
 
-=head1 VERSION
-
-0.501
-
 =head1 NAME
 
 Mojolicious::Plugin::RoutesAuthDBI - from DBI sql-tables does generate routes, make authentication and make restrict access (authorization) to request. Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> on which is based.
@@ -314,8 +357,7 @@ First of all you will see L<SVG|https://github.com/mche/Mojolicious-Plugin-Route
     auth => {...},
     access => {...},
     admin => {...},
-    pos => {...},
-    template => {...},
+    oauth => {...},
   );
 
 
@@ -325,7 +367,7 @@ One option C<dbh> is mandatory, all other - optional.
 
 =head3 dbh
 
-Handler DBI connection where are tables: controllers, actions, routes, users, roles, refs.
+Handler DBI connection where are tables: controllers, actions, routes, logins, profiles, roles, refs.
 
   dbh => $app->dbh,
   # or
@@ -366,7 +408,7 @@ See L<Mojolicious::Plugin::RoutesAuthDBI::Access> for detail options list.
 
 =head3 admin
 
-Hashref options for admin controller for actions on SQL tables routes, roles, users. By default the builtin module:
+Hashref options for admin controller for actions on SQL tables routes, roles, profiles, logins. By default the builtin module:
 
   admin => {
     controller => 'Access',
@@ -385,13 +427,26 @@ You might define your own controller by passing options:
 
 See L<Mojolicious::Plugin::RoutesAuthDBI::Admin> for detail options list.
 
-=head3 pos
+=head3 oauth
 
-Hashref options for POS-dictionary instance. See L<Mojolicious::Plugin::RoutesAuthDBI::Sth>.
+Hashref options for oauth controller. By default the builtin module:
 
-=head3 template
+  oauth => {
+    controller => 'OAuth2',
+    namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
+    ...,
+  },
+
+
+You might define your own controller by passing options:
+
+  oauth => {
+    controller => 'Foo::Bar::Baz',
+    ...,
+  },
+
+See L<Mojolicious::Plugin::RoutesAuthDBI::OAuth2> for detail options list.
 
-Hashref options for template of POS-dictionary. See L<Mojolicious::Plugin::RoutesAuthDBI::POS::Pg>.
 
 =head1 INSTALL
 
@@ -415,7 +470,7 @@ Heart of this plugin! This condition apply for all db routes even if column auth
   # same as
   # $r->route('/foo')->...->over(authenticated => 1)->...; # see Mojolicious::Plugin::Authentication
 
-=item * Route accessible if user roles assigned to either B<loadable> namespace or controller 'Bar.pm' (which assigned neither namespece on db or assigned to that loadable namespace) or action 'bar' on controller Bar.pm (action record in db table actions):
+=item * Route accessible if profile roles assigned to either B<loadable> namespace or controller 'Bar.pm' (which assigned neither namespece on db or assigned to that loadable namespace) or action 'bar' on controller Bar.pm (action record in db table actions):
 
   $r->route('/bar-bar-any-namespace')->to('bar#bar',)->over(access=>{auth=>1})->...;
 
@@ -439,15 +494,15 @@ Heart of this plugin! This condition apply for all db routes even if column auth
 
   $r->route('/bar-cX-aX')->to('bar#bar',)->over(access=>{auth=>1, controller=>'BarX', action=>'barX'})->...;
 
-=item * Route accessible if user roles list has defined role (admin):
+=item * Route accessible if profile roles list has defined role (admin):
 
   $r->route('/bar-role-admin')->to('bar#bar',)->over(access=>{auth=>1, role=> 'admin'})->...;
   
 =item * Pass callback to access condition
 
-The callback will get parameters: $user, $route, $c, $captures, $args (this callback ref). Callback must returns true or false for restrict access. Example simple auth access:
+The callback will get parameters: $profile, $route, $c, $captures, $args (this callback ref). Callback must returns true or false for restrict access. Example simple auth access:
 
-  $r->route('/check-auth')->over(access=>sub {my ($user, $route, $c, $captures, $args) = @_; return $user;})->to(cb=>sub {my $c =shift; $c->render(format=>'txt', text=>"Hi @{[$c->auth_user->{login}]}!\n\nYou have access!");});
+  $r->route('/check-auth')->over(access=>sub {my ($profile, $route, $c, $captures, $args) = @_; return $profile;})->to(cb=>sub {my $c =shift; $c->render(format=>'txt', text=>"Hi @{[$c->auth_user->{names}]}!\n\nYou have access!");});
 
 =back
 
